from helper import ExtendedEnum
from messagebus import TokenCreator, TokenPriority

PERIPHERALS_GROUP = "6"


class ListenedQueueTokens(ExtendedEnum):
    TK_PRN_PRINT = TokenCreator.create_token(TokenPriority.high, PERIPHERALS_GROUP, "101")  # 4032823553
    TK_PRN_GETSTATUS = TokenCreator.create_token(TokenPriority.high, PERIPHERALS_GROUP, "102")
    TK_CDRAWER_OPEN = TokenCreator.create_token(TokenPriority.high, PERIPHERALS_GROUP, "201")
    TK_CDRAWER_GETSTATUS = TokenCreator.create_token(TokenPriority.high, PERIPHERALS_GROUP, "202")
