from typing import Any

from messageprocessorutil import UuidMessageProcessor

from nperipherals_core.interactor import HealthCheckInteractor


class HealthCheckProcessor(UuidMessageProcessor):
    def __init__(self, interactor):
        # type: (HealthCheckInteractor) -> None

        super(HealthCheckProcessor, self).__init__()
        self.interactor = interactor

    def parse_data(self, data, event_type):
        # type: (Any, Any) -> Any

        return

    def call_business(self, obj):
        # type: (Any) -> None
        return self.interactor.restart_dead_threads()

    def format_response(self, message_bus, message, event, input_obj, result):
        return

    def format_exception(self, message_bus, message, event, input_obj, exception):
        return

    def format_parse_exception(self, message_bus, message, event, data, exception):
        return
