from logging import getLogger

from nperipherals_core.model.repository import NotificationRepository

logger = getLogger(__name__)


class LoggerNotificationRepository(NotificationRepository):
    def notify(self, name, data):
        # type: (str, str) -> None

        logger.info("Printing data [{}] from device [{}]".format(data, name))
