from typing import List

from ._DeliveryHeaderDto import DeliveryHeaderDto
from ._DeliveryBodyDto import DeliveryBodyDto


class DeliveryGeneratorDto(object):
    def __init__(self, header, body, products, cfh_products):
        # type: (DeliveryHeaderDto, DeliveryBodyDto, object, List) -> ()
        self.header = header
        self.body = body
        self.products = products
        self.cfh_products = cfh_products
