class SaleLine(object):
    def __init__(self, line_number, item_id, level, part_code):
        self.line_number = line_number
        self.item_id = item_id
        self.level = level
        self.part_code = part_code
